#
# Routing / IP rule tests for netplan generator
#
# Copyright (C) 2018 Canonical, Ltd.
# Author: Mathieu Trudel-Lapierre <mathieu.trudel.lapierre@canonical.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .base import TestBase, ND_VLAN, ND_DHCP4, ND_EMPTY


class TestNetworkd(TestBase):

    def test_route_invalid_family_to(self):
        err = self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: abc/24
          via: 192.168.14.20''', expect_fail=True)
        self.assertIn("Error in network definition: invalid IP family '-1'", err)

    def test_route_v4_single(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          metric: 100
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
Metric=100
'''})

    def test_route_v4_single_mulit_parse(self):
        self.generate('''network:
  version: 2
  bridges:
    br0: {interfaces: [engreen]}
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          metric: 100
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=no
Address=192.168.14.2/24
Bridge=br0

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
Metric=100
''',
                              'br0.netdev': '[NetDev]\nName=br0\nKind=bridge\n',
                              'br0.network': '''[Match]\nName=br0\n
[Network]
LinkLocalAddressing=ipv6
ConfigureWithoutCarrier=yes
'''})

    def test_route_v4_multiple(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 8.8.0.0/16
          via: 192.168.1.1
        - to: 10.10.10.8
          via: 192.168.1.2
          metric: 5000
        - to: 11.11.11.0/24
          via: 192.168.1.3
          metric: 9999
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=8.8.0.0/16
Gateway=192.168.1.1

[Route]
Destination=10.10.10.8
Gateway=192.168.1.2
Metric=5000

[Route]
Destination=11.11.11.0/24
Gateway=192.168.1.3
Metric=9999
'''})

    def test_route_v4_default(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.1.2/24"]
      routes:
        - to: default
          via: 192.168.1.1
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.1.2/24

[Route]
Destination=0.0.0.0/0
Gateway=192.168.1.1
'''})

    def test_route_v4_onlink(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          on-link: true
          metric: 100
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
GatewayOnLink=true
Metric=100
'''})

    def test_route_v4_onlink_no(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          on-link: n
          metric: 100
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
Metric=100
'''})

    def test_route_v4_scope(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          scope: link
          metric: 100
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Scope=link
Metric=100
'''})

    def test_route_v4_scope_redefine(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          scope: host
          via: 192.168.14.20
          scope: link
          metric: 100
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
Scope=link
Metric=100
'''})

    def test_route_v4_type_blackhole(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          type: blackhole
          metric: 100
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
Type=blackhole
Metric=100
'''})

    def test_route_v4_type_redefine(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          type: prohibit
          via: 192.168.14.20
          type: unicast
          metric: 100
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
Metric=100
'''})

    def test_route_v4_table(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          table: 201
          metric: 100
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
Metric=100
Table=201
'''})

    def test_route_v4_from(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          from: 192.168.14.2
          metric: 100
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
PreferredSource=192.168.14.2
Metric=100
'''})

    def test_route_v4_mtu(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          mtu: 1500
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
MTUBytes=1500
'''})

    def test_route_v4_congestion_window(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          congestion-window: 16
        ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
InitialCongestionWindow=16
'''})

    def test_route_v4_advertised_receive_window(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          advertised-receive-window: 16
        ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
InitialAdvertisedReceiveWindow=16
'''})

    def test_route_v6_single(self):
        self.generate('''network:
  version: 2
  ethernets:
    enblue:
      addresses: ["192.168.1.3/24"]
      routes:
        - to: 2001:dead:beef::2/64
          via: 2001:beef:beef::1''')

        self.assert_networkd({'enblue.network': '''[Match]
Name=enblue

[Network]
LinkLocalAddressing=ipv6
Address=192.168.1.3/24

[Route]
Destination=2001:dead:beef::2/64
Gateway=2001:beef:beef::1
'''})

    def test_route_v6_type(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 2001:dead:beef::2/64
          via: 2001:beef:beef::1
          type: prohibit''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=2001:dead:beef::2/64
Gateway=2001:beef:beef::1
Type=prohibit
'''})

    def test_route_v6_scope_host(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 2001:dead:beef::2/64
          via: 2001:beef:beef::1
          scope: host''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=2001:dead:beef::2/64
Gateway=2001:beef:beef::1
Scope=host
'''})

    def test_route_v6_multiple(self):
        self.generate('''network:
  version: 2
  ethernets:
    enblue:
      addresses: ["192.168.1.3/24"]
      routes:
        - to: 2001:dead:beef::2/64
          via: 2001:beef:beef::1
        - to: 2001:f00f:f00f::fe/64
          via: 2001:beef:feed::1
          metric: 1024''')

        self.assert_networkd({'enblue.network': '''[Match]
Name=enblue

[Network]
LinkLocalAddressing=ipv6
Address=192.168.1.3/24

[Route]
Destination=2001:dead:beef::2/64
Gateway=2001:beef:beef::1

[Route]
Destination=2001:f00f:f00f::fe/64
Gateway=2001:beef:feed::1
Metric=1024
'''})

    def test_route_v6_default(self):
        self.generate('''network:
  version: 2
  ethernets:
    enblue:
      addresses: ["2001:dead:beef::2/64"]
      routes:
        - to: default
          via: 2001:beef:beef::1''')

        self.assert_networkd({'enblue.network': '''[Match]
Name=enblue

[Network]
LinkLocalAddressing=ipv6
Address=2001:dead:beef::2/64

[Route]
Destination=::/0
Gateway=2001:beef:beef::1
'''})

    def test_ip_rule_table(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routing-policy:
        - to: 10.10.10.0/24
          table: 100
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[RoutingPolicyRule]
To=10.10.10.0/24
Table=100
'''})

    def test_ip_rule_priority(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routing-policy:
        - to: 10.10.10.0/24
          priority: 99
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[RoutingPolicyRule]
To=10.10.10.0/24
Priority=99
'''})

    def test_ip_rule_fwmark(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routing-policy:
        - from: 10.10.10.0/24
          mark: 50
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[RoutingPolicyRule]
From=10.10.10.0/24
FirewallMark=50
'''})

    def test_ip_rule_tos(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routing-policy:
        - to: 10.10.10.0/24
          type-of-service: 250
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[RoutingPolicyRule]
To=10.10.10.0/24
TypeOfService=250
'''})

    def test_use_routes(self):
        """[networkd] Validate config generation when use-routes DHCP override is used"""
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: true
      dhcp4-overrides:
        use-routes: false
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
DHCP=ipv4
LinkLocalAddressing=ipv6

[DHCP]
RouteMetric=100
UseMTU=true
UseRoutes=false
'''})

    def test_default_metric(self):
        """[networkd] Validate config generation when metric DHCP override is used"""
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: true
      dhcp6: true
      dhcp4-overrides:
        route-metric: 3333
      dhcp6-overrides:
        route-metric: 3333
    enred:
      dhcp4: true
      dhcp6: true
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
DHCP=yes
LinkLocalAddressing=ipv6

[DHCP]
RouteMetric=3333
UseMTU=true
''',
                              'enred.network': '''[Match]
Name=enred

[Network]
DHCP=yes
LinkLocalAddressing=ipv6

[DHCP]
RouteMetric=100
UseMTU=true
'''})

    def test_default_scope_link_lp1805038(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: true
      routes:
      - to: 10.96.0.0/24
    enred:
      dhcp4: true
      routes:
      - to: 10.97.0.0/24
        type: broadcast
''', skip_generated_yaml_validation=True)  # scope: link is a default value in this case

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
DHCP=ipv4
LinkLocalAddressing=ipv6

[Route]
Destination=10.96.0.0/24
Scope=link

[DHCP]
RouteMetric=100
UseMTU=true
''',
                              'enred.network': '''[Match]
Name=enred

[Network]
DHCP=ipv4
LinkLocalAddressing=ipv6

[Route]
Destination=10.97.0.0/24
Scope=link
Type=broadcast

[DHCP]
RouteMetric=100
UseMTU=true
'''})

    def test_type_local_lp1892272(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: true
      routes:
      - to: 0.0.0.0/0
        type: local
        table: 99
      - to: ::0/0
        type: local
        table: 100
''', skip_generated_yaml_validation=True)  # scope: host is a default value in this case

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
DHCP=ipv4
LinkLocalAddressing=ipv6

[Route]
Destination=0.0.0.0/0
Scope=host
Type=local
Table=99

[Route]
Destination=::0/0
Scope=host
Type=local
Table=100

[DHCP]
RouteMetric=100
UseMTU=true
'''})

    def test_route_metric_rendering_lp2023681(self):
        """Validate metric rendering is unsigned
        (can render up to 4294967294, 4294967295 is used internally to define an unset value)
        """

        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          metric: 4294967294
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
Metric=4294967294
'''})

    def test_route_v4_advmss_systemd(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          advertised-mss: 1400
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
TCPAdvertisedMaximumSegmentSize=1400
'''})

    def test_route_v4_advmss_empty_systemd(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          ''')

        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=ipv6
Address=192.168.14.2/24

[Route]
Destination=10.10.10.0/24
Gateway=192.168.14.20
'''})


class TestNetworkManager(TestBase):

    def test_route_v4_single(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          metric: 100
          ''')

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
route1=10.10.10.0/24,192.168.14.20,100

[ipv6]
method=ignore
'''})

    def test_route_v4_multiple(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 8.8.0.0/16
          via: 192.168.1.1
          metric: 5000
        - to: 10.10.10.8
          via: 192.168.1.2
        - to: 11.11.11.0/24
          via: 192.168.1.3
          metric: 9999
          ''')

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
route1=8.8.0.0/16,192.168.1.1,5000
route2=10.10.10.8,192.168.1.2
route3=11.11.11.0/24,192.168.1.3,9999

[ipv6]
method=ignore
'''})

    def test_route_v4_default(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      addresses: ["192.168.1.2/24"]
      routes:
        - to: default
          via: 192.168.1.1
          ''')

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.1.2/24
route1=0.0.0.0/0,192.168.1.1

[ipv6]
method=ignore
'''})

    def test_route_v6_single(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    enblue:
      addresses: ["2001:f00f:f00f::2/64"]
      routes:
        - to: 2001:dead:beef::2/64
          via: 2001:beef:beef::1''')

        self.assert_nm({'enblue': '''[connection]
id=netplan-enblue
type=ethernet
interface-name=enblue

[ethernet]
wake-on-lan=0

[ipv4]
method=link-local

[ipv6]
method=manual
address1=2001:f00f:f00f::2/64
ip6-privacy=0
route1=2001:dead:beef::2/64,2001:beef:beef::1
'''})

    def test_route_v6_multiple(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    enblue:
      addresses: ["2001:f00f:f00f::2/64"]
      routes:
        - to: 2001:dead:beef::2/64
          via: 2001:beef:beef::1
        - to: 2001:dead:feed::2/64
          via: 2001:beef:beef::2
          metric: 1000''')

        self.assert_nm({'enblue': '''[connection]
id=netplan-enblue
type=ethernet
interface-name=enblue

[ethernet]
wake-on-lan=0

[ipv4]
method=link-local

[ipv6]
method=manual
address1=2001:f00f:f00f::2/64
ip6-privacy=0
route1=2001:dead:beef::2/64,2001:beef:beef::1
route2=2001:dead:feed::2/64,2001:beef:beef::2,1000
'''})

    def test_route_v6_default(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    enblue:
      addresses: ["2001:dead:beef::2/64"]
      routes:
        - to: default
          via: 2001:beef:beef::1''')

        self.assert_nm({'enblue': '''[connection]
id=netplan-enblue
type=ethernet
interface-name=enblue

[ethernet]
wake-on-lan=0

[ipv4]
method=link-local

[ipv6]
method=manual
address1=2001:dead:beef::2/64
ip6-privacy=0
route1=::/0,2001:beef:beef::1
'''})

    def test_ip_rule_missing_priority_fails_ipv4(self):
        err = self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routing-policy:
        - to: 10.10.10.0/24
          table: 100
          ''', expect_fail=True)
        self.assertIn("ERROR: engreen: The priority setting is mandatory for NetworkManager routing-policy", err)

    def test_ip_rule_missing_priority_fails_ipv6(self):
        err = self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      addresses: ["2001:FFfe::1/64"]
      routing-policy:
        - to: 2001:FFfe::1/64
          table: 100
          ''', expect_fail=True)
        self.assertIn("ERROR: engreen: The priority setting is mandatory for NetworkManager routing-policy", err)

    def test_ip_rule_table(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routing-policy:
        - to: 10.10.10.0/24
          table: 100
          priority: 99
          ''')

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
routing-rule1=priority 99 to 10.10.10.0/24 table 100

[ipv6]
method=ignore
'''})

    def test_ip_rule_fwmark(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routing-policy:
        - from: 10.10.10.0/24
          mark: 50
          priority: 99
          ''')

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
routing-rule1=priority 99 from 10.10.10.0/24 fwmark 50

[ipv6]
method=ignore
'''})

    def test_ip_rule_tos(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routing-policy:
        - to: 10.10.10.0/24
          type-of-service: 250
          priority: 99
          ''')

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
routing-rule1=priority 99 to 10.10.10.0/24 tos 250

[ipv6]
method=ignore
'''})

    def test_routes_mixed(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24", "2001:f00f::2/128"]
      routes:
        - to: 2001:dead:beef::2/64
          via: 2001:beef:beef::1
          metric: 997
        - to: 8.8.0.0/16
          via: 192.168.1.1
          metric: 5000
        - to: 10.10.10.8
          via: 192.168.1.2
        - to: 11.11.11.0/24
          via: 192.168.1.3
          metric: 9999
        - to: 2001:f00f:f00f::fe/64
          via: 2001:beef:feed::1
          ''')

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
route1=8.8.0.0/16,192.168.1.1,5000
route2=10.10.10.8,192.168.1.2
route3=11.11.11.0/24,192.168.1.3,9999

[ipv6]
method=manual
address1=2001:f00f::2/128
ip6-privacy=0
route1=2001:dead:beef::2/64,2001:beef:beef::1,997
route2=2001:f00f:f00f::fe/64,2001:beef:feed::1
'''})

    def test_route_from(self):
        out = self.generate('''network:
  version: 2
  ethernets:
    engreen:
      renderer: NetworkManager
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          from: 192.168.14.2
          ''')
        self.assertEqual('', out)

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
route1=10.10.10.0/24,192.168.14.20
route1_options=src=192.168.14.2

[ipv6]
method=ignore
'''})
        self.assert_networkd({})

    def test_route_onlink(self):
        out = self.generate('''network:
  version: 2
  ethernets:
    engreen:
      renderer: NetworkManager
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.1.20
          on-link: true
          ''')
        self.assertEqual('', out)

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
route1=10.10.10.0/24,192.168.1.20
route1_options=onlink=true

[ipv6]
method=ignore
'''})
        self.assert_networkd({})

    def test_route_table(self):
        out = self.generate('''network:
  version: 2
  ethernets:
    engreen:
      renderer: NetworkManager
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.1.20
          table: 31337
          ''')
        self.assertEqual('', out)

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
route1=10.10.10.0/24,192.168.1.20
route1_options=table=31337

[ipv6]
method=ignore
'''})
        self.assert_networkd({})

    def test_route_mtu(self):
        out = self.generate('''network:
  version: 2
  ethernets:
    engreen:
      renderer: NetworkManager
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.1.20
          mtu: 1500
          ''')
        self.assertEqual('', out)

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
route1=10.10.10.0/24,192.168.1.20
route1_options=mtu=1500

[ipv6]
method=ignore
'''})
        self.assert_networkd({})

    def test_route_congestion_window(self):
        out = self.generate('''network:
  version: 2
  ethernets:
    engreen:
      renderer: NetworkManager
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.1.20
          congestion-window: 16
        ''')
        self.assertEqual('', out)

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
route1=10.10.10.0/24,192.168.1.20
route1_options=initcwnd=16

[ipv6]
method=ignore
'''})
        self.assert_networkd({})

    def test_route_advertised_receive_window(self):
        out = self.generate('''network:
  version: 2
  ethernets:
    engreen:
      renderer: NetworkManager
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.1.20
          advertised-receive-window: 16
        ''')
        self.assertEqual('', out)

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
route1=10.10.10.0/24,192.168.1.20
route1_options=initrwnd=16

[ipv6]
method=ignore
'''})
        self.assert_networkd({})

    def test_route_options(self):
        out = self.generate('''network:
  version: 2
  ethernets:
    engreen:
      renderer: NetworkManager
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.1.20
          table: 31337
          from: 192.168.14.2
          on-link: true
          ''')
        self.assertEqual('', out)

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
route1=10.10.10.0/24,192.168.1.20
route1_options=onlink=true,table=31337,src=192.168.14.2

[ipv6]
method=ignore
'''})
        self.assert_networkd({})

    def test_route_reject_type(self):
        err = self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.1.20
          type: blackhole
          ''', expect_fail=True)
        self.assertIn('NetworkManager only supports unicast routes', err)

        self.assert_nm({})
        self.assert_networkd({})

    def test_route_reject_type_v6(self):
        err = self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      addresses: ["2001:f00f::2/128"]
      routes:
        - to: 2001:dead:beef::2/64
          via: 2001:beef:beef::1
          type: blackhole
          ''', expect_fail=True)
        self.assertIn('NetworkManager only supports unicast routes', err)

        self.assert_nm({})
        self.assert_networkd({})

    def test_use_routes_v4(self):
        """[NetworkManager] Validate config when use-routes DHCP4 override is used"""
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      dhcp4: true
      dhcp4-overrides:
        use-routes: false
          ''')
        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=auto
ignore-auto-routes=true
never-default=true

[ipv6]
method=ignore
'''})

    def test_use_routes_v6(self):
        """[NetworkManager] Validate config when use-routes DHCP6 override is used"""
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      dhcp4: true
      dhcp6: true
      dhcp6-overrides:
        use-routes: false
          ''')
        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=auto

[ipv6]
method=auto
ip6-privacy=0
ignore-auto-routes=true
never-default=true
'''})

    def test_default_metric_v4(self):
        """[NetworkManager] Validate config when setting a default metric for DHCPv4"""
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      dhcp4: true
      dhcp6: true
      dhcp4-overrides:
        route-metric: 4000
          ''')
        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=auto
route-metric=4000

[ipv6]
method=auto
ip6-privacy=0
'''})

    def test_default_metric_v6(self):
        """[NetworkManager] Validate config when setting a default metric for DHCPv6"""
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      dhcp4: true
      dhcp6: true
      dhcp6-overrides:
        route-metric: 5050
          ''')
        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=auto

[ipv6]
method=auto
ip6-privacy=0
route-metric=5050
'''})

    def test_add_routes_to_different_tables_from_multiple_files(self):
        """Test case for bug LP#2003061"""

        self.generate('''network:
  version: 2''',
                      confs={'01-netcfg': '''network:
  version: 2
  ethernets:
    eth0:
      dhcp4: true
  vlans:
    vlan100:
      id: 100
      link: eth0''',
                             '10-table1': '''network:
  version: 2
  ethernets: {eth0: {dhcp4: true}}
  vlans:
    vlan100:
      id: 100
      link: eth0
      routing-policy:
        - from: 10.0.0.1
          table: 1001
      routes:
        - to: 0.0.0.0/0
          via: 10.0.0.100
          table: 1001''',
                             '10-table2': '''network:
  version: 2
  ethernets: {eth0: {dhcp4: true}}
  vlans:
    vlan100:
      id: 100
      link: eth0
      routing-policy:
        - from: 10.0.0.2
          table: 1002
      routes:
        - to: 0.0.0.0/0
          via: 10.0.0.200
          table: 1002'''})

        self.assert_networkd({'eth0.network': (ND_DHCP4 % 'eth0').replace('\n[DHCP]', 'VLAN=vlan100\n\n[DHCP]'),
                              'vlan100.netdev': ND_VLAN % ('vlan100', 100),
                              'vlan100.network': ND_EMPTY % ('vlan100', 'ipv6') + '''
[Route]
Destination=0.0.0.0/0
Gateway=10.0.0.100
Table=1001

[Route]
Destination=0.0.0.0/0
Gateway=10.0.0.200
Table=1002

[RoutingPolicyRule]
From=10.0.0.1
Table=1001

[RoutingPolicyRule]
From=10.0.0.2
Table=1002
'''})

    def test_add_duplicate_routes_from_multiple_files(self):
        """ Duplicate route should produce a single entry in the
        backend configuration"""

        self.generate('''network:
  version: 2''',
                      confs={'01-netcfg': '''network:
  version: 2
  ethernets:
    eth0:
      dhcp4: true
  vlans:
    vlan100:
      id: 100
      link: eth0''',
                             '10-table1': '''network:
  version: 2
  ethernets: {eth0: {dhcp4: true}}
  vlans:
    vlan100:
      id: 100
      link: eth0
      routing-policy:
        - from: 10.0.0.1
          table: 1001
      routes:
        - to: 0.0.0.0/0
          via: 10.0.0.100
          table: 1001''',
                             '10-table2': '''network:
  version: 2
  ethernets: {eth0: {dhcp4: true}}
  vlans:
    vlan100:
      id: 100
      link: eth0
      routing-policy:
        - from: 10.0.0.2
          table: 1002
      routes:
        - to: 0.0.0.0/0
          via: 10.0.0.100
          table: 1001'''})

        self.assert_networkd({'eth0.network': (ND_DHCP4 % 'eth0').replace('\n[DHCP]', 'VLAN=vlan100\n\n[DHCP]'),
                              'vlan100.netdev': ND_VLAN % ('vlan100', 100),
                              'vlan100.network': ND_EMPTY % ('vlan100', 'ipv6') + '''
[Route]
Destination=0.0.0.0/0
Gateway=10.0.0.100
Table=1001

[RoutingPolicyRule]
From=10.0.0.1
Table=1001

[RoutingPolicyRule]
From=10.0.0.2
Table=1002
'''})

    def test_route_metric_rendering_lp2023681(self):
        """Validate metric rendering is unsigned
        (can render up to 4294967294, 4294967295 is used internally to define an unset value)
        """

        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          metric: 4294967294
          ''')

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
route1=10.10.10.0/24,192.168.14.20,4294967294

[ipv6]
method=ignore
'''})

    def test_route_v4_advmss_nm(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      addresses: ["192.168.14.2/24"]
      routes:
        - to: 10.10.10.0/24
          via: 192.168.14.20
          advertised-mss: 1400
          ''')

        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=manual
address1=192.168.14.2/24
route1=10.10.10.0/24,192.168.14.20
route1_options=advmss=1400

[ipv6]
method=ignore
'''})



#
# Tests for bond devices config generated via netplan
#
# Copyright (C) 2018 Canonical, Ltd.
# Author: Mathieu Trudel-Lapierre <mathieu.trudel.lapierre@canonical.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .base import TestBase, NM_MANAGED, NM_UNMANAGED


class TestNetworkd(TestBase):

    def test_bond_dhcp6_no_accept_ra(self):
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp6: no
      accept-ra: no
  bonds:
    bond0:
      interfaces: [engreen]
      dhcp6: true
      accept-ra: yes''')
        self.assert_networkd({'bond0.network': '''[Match]
Name=bond0

[Network]
DHCP=ipv6
LinkLocalAddressing=ipv6
IPv6AcceptRA=yes
ConfigureWithoutCarrier=yes

[DHCP]
RouteMetric=100
UseMTU=true
''',
                              'bond0.netdev': '''[NetDev]
Name=bond0
Kind=bond
''',
                              'engreen.network': '''[Match]
Name=engreen

[Network]
LinkLocalAddressing=no
IPv6AcceptRA=no
Bond=bond0
'''})

    def test_bond_empty(self):
        self.generate('''network:
  version: 2
  bonds:
    bn0:
      dhcp4: true''')

        self.assert_networkd({'bn0.netdev': '[NetDev]\nName=bn0\nKind=bond\n',
                              'bn0.network': '''[Match]
Name=bn0

[Network]
DHCP=ipv4
LinkLocalAddressing=ipv6
ConfigureWithoutCarrier=yes

[DHCP]
RouteMetric=100
UseMTU=true
'''})
        self.assert_nm(None)
        self.assert_nm_udev(NM_UNMANAGED % 'bn0')

    def test_bond_components(self):
        self.generate('''network:
  version: 2
  ethernets:
    eno1: {}
    switchports:
      match:
        driver: yayroute
  bonds:
    bn0:
      interfaces: [eno1, switchports]
      dhcp4: true''')

        self.assert_networkd({'bn0.netdev': '[NetDev]\nName=bn0\nKind=bond\n',
                              'bn0.network': '''[Match]
Name=bn0

[Network]
DHCP=ipv4
LinkLocalAddressing=ipv6
ConfigureWithoutCarrier=yes

[DHCP]
RouteMetric=100
UseMTU=true
''',
                              'eno1.network': '[Match]\nName=eno1\n\n'
                                              '[Network]\nLinkLocalAddressing=no\nBond=bn0\n',
                              'switchports.network': '[Match]\nDriver=yayroute\n\n'
                                                     '[Network]\nLinkLocalAddressing=no\nBond=bn0\n'})

    def test_bond_empty_parameters(self):
        self.generate('''network:
  version: 2
  ethernets:
    eno1: {}
    switchports:
      match:
        driver: yayroute
  bonds:
    bn0:
      parameters: {}
      interfaces: [eno1, switchports]
      dhcp4: true''')

        self.assert_networkd({'bn0.netdev': '[NetDev]\nName=bn0\nKind=bond\n',
                              'bn0.network': '''[Match]
Name=bn0

[Network]
DHCP=ipv4
LinkLocalAddressing=ipv6
ConfigureWithoutCarrier=yes

[DHCP]
RouteMetric=100
UseMTU=true
''',
                              'eno1.network': '[Match]\nName=eno1\n\n'
                                              '[Network]\nLinkLocalAddressing=no\nBond=bn0\n',
                              'switchports.network': '[Match]\nDriver=yayroute\n\n'
                                                     '[Network]\nLinkLocalAddressing=no\nBond=bn0\n'})

    def test_bond_with_parameters_all_members_active(self):
        self.generate('''network:
  version: 2
  ethernets:
    eno1: {}
    switchports:
      match:
        driver: yayroute
  bonds:
    bn0:
      parameters:
        mode: 802.3ad
        lacp-rate: fast
        mii-monitor-interval: 10
        min-links: 10
        up-delay: 20
        down-delay: 30
        all-members-active: true
        transmit-hash-policy: none
        ad-select: none
        arp-interval: 15
        arp-validate: all
        arp-all-targets: all
        fail-over-mac-policy: none
        gratuitious-arp: 10
        packets-per-member: 10
        primary-reselect-policy: none
        resend-igmp: 10
        learn-packet-interval: 10
        arp-ip-targets:
          - 10.10.10.10
          - 20.20.20.20
      interfaces: [eno1, switchports]
      dhcp4: true''')

        self.assert_networkd({'bn0.netdev': '[NetDev]\nName=bn0\nKind=bond\n\n'
                                            '[Bond]\n'
                                            'Mode=802.3ad\n'
                                            'LACPTransmitRate=fast\n'
                                            'MIIMonitorSec=10ms\n'
                                            'MinLinks=10\n'
                                            'TransmitHashPolicy=none\n'
                                            'AdSelect=none\n'
                                            'AllSlavesActive=1\n'  # wokeignore:rule=slave
                                            'ARPIntervalSec=15ms\n'
                                            'ARPIPTargets=10.10.10.10 20.20.20.20\n'
                                            'ARPValidate=all\n'
                                            'ARPAllTargets=all\n'
                                            'UpDelaySec=20ms\n'
                                            'DownDelaySec=30ms\n'
                                            'FailOverMACPolicy=none\n'
                                            'GratuitousARP=10\n'
                                            'PacketsPerSlave=10\n'  # wokeignore:rule=slave
                                            'PrimaryReselectPolicy=none\n'
                                            'ResendIGMP=10\n'
                                            'LearnPacketIntervalSec=10\n',
                              'bn0.network': '''[Match]
Name=bn0

[Network]
DHCP=ipv4
LinkLocalAddressing=ipv6
ConfigureWithoutCarrier=yes

[DHCP]
RouteMetric=100
UseMTU=true
''',
                              'eno1.network': '[Match]\nName=eno1\n\n'
                                              '[Network]\nLinkLocalAddressing=no\nBond=bn0\n',
                              'switchports.network': '[Match]\nDriver=yayroute\n\n'
                                                     '[Network]\nLinkLocalAddressing=no\nBond=bn0\n'})

    def test_bond_with_parameters(self):
        self.generate('''network:
  version: 2
  ethernets:
    eno1: {}
    switchports:
      match:
        driver: yayroute
  bonds:
    bn0:
      parameters:
        mode: 802.3ad
        lacp-rate: fast
        mii-monitor-interval: 10
        min-links: 10
        up-delay: 20
        down-delay: 30
        all-slaves-active: true  # wokeignore:rule=slave
        transmit-hash-policy: none
        ad-select: none
        arp-interval: 15
        arp-validate: all
        arp-all-targets: all
        fail-over-mac-policy: none
        gratuitious-arp: 10
        packets-per-slave: 10  # wokeignore:rule=slave
        primary-reselect-policy: none
        resend-igmp: 10
        learn-packet-interval: 10
        arp-ip-targets:
          - 10.10.10.10
          - 20.20.20.20
      interfaces: [eno1, switchports]
      dhcp4: true''', skip_generated_yaml_validation=True)
        # Skipping the yaml validation above because the emitter will use
        # all-members-active and packets-per-member by default.

        self.assert_networkd({'bn0.netdev': '[NetDev]\nName=bn0\nKind=bond\n\n'
                                            '[Bond]\n'
                                            'Mode=802.3ad\n'
                                            'LACPTransmitRate=fast\n'
                                            'MIIMonitorSec=10ms\n'
                                            'MinLinks=10\n'
                                            'TransmitHashPolicy=none\n'
                                            'AdSelect=none\n'
                                            'AllSlavesActive=1\n'  # wokeignore:rule=slave
                                            'ARPIntervalSec=15ms\n'
                                            'ARPIPTargets=10.10.10.10 20.20.20.20\n'
                                            'ARPValidate=all\n'
                                            'ARPAllTargets=all\n'
                                            'UpDelaySec=20ms\n'
                                            'DownDelaySec=30ms\n'
                                            'FailOverMACPolicy=none\n'
                                            'GratuitousARP=10\n'
                                            'PacketsPerSlave=10\n'  # wokeignore:rule=slave
                                            'PrimaryReselectPolicy=none\n'
                                            'ResendIGMP=10\n'
                                            'LearnPacketIntervalSec=10\n',
                              'bn0.network': '''[Match]
Name=bn0

[Network]
DHCP=ipv4
LinkLocalAddressing=ipv6
ConfigureWithoutCarrier=yes

[DHCP]
RouteMetric=100
UseMTU=true
''',
                              'eno1.network': '[Match]\nName=eno1\n\n'
                                              '[Network]\nLinkLocalAddressing=no\nBond=bn0\n',
                              'switchports.network': '[Match]\nDriver=yayroute\n\n'
                                                     '[Network]\nLinkLocalAddressing=no\nBond=bn0\n'})

    def test_bond_with_parameters_all_suffix(self):
        self.generate('''network:
  version: 2
  ethernets:
    eno1: {}
    switchports:
      match:
        driver: yayroute
  bonds:
    bn0:
      parameters:
        mode: 802.3ad
        mii-monitor-interval: 10ms
        up-delay: 20ms
        down-delay: 30s
        arp-interval: 15m
      interfaces: [eno1, switchports]
      dhcp4: true''')

        self.assert_networkd({'bn0.netdev': '[NetDev]\nName=bn0\nKind=bond\n\n'
                                            '[Bond]\n'
                                            'Mode=802.3ad\n'
                                            'MIIMonitorSec=10ms\n'
                                            'ARPIntervalSec=15m\n'
                                            'UpDelaySec=20ms\n'
                                            'DownDelaySec=30s\n',
                              'bn0.network': '''[Match]
Name=bn0

[Network]
DHCP=ipv4
LinkLocalAddressing=ipv6
ConfigureWithoutCarrier=yes

[DHCP]
RouteMetric=100
UseMTU=true
''',
                              'eno1.network': '[Match]\nName=eno1\n\n'
                                              '[Network]\nLinkLocalAddressing=no\nBond=bn0\n',
                              'switchports.network': '[Match]\nDriver=yayroute\n\n'
                                                     '[Network]\nLinkLocalAddressing=no\nBond=bn0\n'})

    def test_bond_primary_member(self):
        self.generate('''network:
  version: 2
  ethernets:
    eno1: {}
    switchports:
      match:
        driver: yayroute
  bonds:
    bn0:
      parameters:
        mode: active-backup
        primary: eno1
      interfaces: [eno1, switchports]
      dhcp4: true''')

        self.assert_networkd({'bn0.netdev': '[NetDev]\nName=bn0\nKind=bond\n\n'
                                            '[Bond]\n'
                                            'Mode=active-backup\n',
                              'bn0.network': '''[Match]
Name=bn0

[Network]
DHCP=ipv4
LinkLocalAddressing=ipv6
ConfigureWithoutCarrier=yes

[DHCP]
RouteMetric=100
UseMTU=true
''',
                              'eno1.network': '[Match]\nName=eno1\n\n'
                                              '[Network]\nLinkLocalAddressing=no\nBond=bn0\n'
                                              'PrimarySlave=true\n',  # wokeignore:rule=slave
                              'switchports.network': '[Match]\nDriver=yayroute\n\n'
                                                     '[Network]\nLinkLocalAddressing=no\nBond=bn0\n'})

    def test_bond_primary_member_duplicate(self):
        self.generate('''network:
  version: 2
  renderer: networkd
  ethernets:
    eno1: {}
    enp65s0: {}
    faketh2: {}
  bonds:
    bond0:
      interfaces: [eno1, enp65s0]
      parameters:
        primary: enp65s0
        mode: balance-tlb
  vlans:
    vbr-v10:
      id: 10
      link: vbr
  bridges:
    vbr:
      interfaces: [faketh2]''', expect_fail=False)

        self.assert_networkd({'eno1.network': '[Match]\nName=eno1\n\n[Network]\nLinkLocalAddressing=no\nBond=bond0\n',
                              'enp65s0.network': '''[Match]
Name=enp65s0

[Network]
LinkLocalAddressing=no
Bond=bond0
PrimarySlave=true # wokeignore:rule=slave
''',
                              'faketh2.network': '[Match]\nName=faketh2\n\n[Network]\nLinkLocalAddressing=no\nBridge=vbr\n',
                              'bond0.network': '[Match]\nName=bond0\n\n'
                                               '[Network]\nLinkLocalAddressing=ipv6\nConfigureWithoutCarrier=yes\n',
                              'bond0.netdev': '[NetDev]\nName=bond0\nKind=bond\n\n[Bond]\nMode=balance-tlb\n',
                              'vbr-v10.network': '[Match]\nName=vbr-v10\n\n'
                                                 '[Network]\nLinkLocalAddressing=ipv6\nConfigureWithoutCarrier=yes\n',
                              'vbr-v10.netdev': '[NetDev]\nName=vbr-v10\nKind=vlan\n\n[VLAN]\nId=10\n',
                              'vbr.network': '[Match]\nName=vbr\n\n'
                                             '[Network]\nLinkLocalAddressing=ipv6\nConfigureWithoutCarrier=yes\nVLAN=vbr-v10\n',
                              'vbr.netdev': '[NetDev]\nName=vbr\nKind=bridge\n'})

    def test_bond_with_gratuitous_spelling(self):
        """Validate that the correct spelling of gratuitous also works"""
        self.generate('''network:
  version: 2
  ethernets:
    eno1: {}
    switchports:
      match:
        driver: yayroute
  bonds:
    bn0:
      parameters:
        mode: active-backup
        gratuitous-arp: 10
      interfaces: [eno1, switchports]
      dhcp4: true''')

        self.assert_networkd({'bn0.netdev': '[NetDev]\nName=bn0\nKind=bond\n\n'
                                            '[Bond]\n'
                                            'Mode=active-backup\n'
                                            'GratuitousARP=10\n',
                              'bn0.network': '''[Match]
Name=bn0

[Network]
DHCP=ipv4
LinkLocalAddressing=ipv6
ConfigureWithoutCarrier=yes

[DHCP]
RouteMetric=100
UseMTU=true
''',
                              'eno1.network': '[Match]\nName=eno1\n\n'
                                              '[Network]\nLinkLocalAddressing=no\nBond=bn0\n',
                              'switchports.network': '[Match]\nDriver=yayroute\n\n'
                                                     '[Network]\nLinkLocalAddressing=no\nBond=bn0\n'})


class TestNetworkManager(TestBase):

    def test_bond_empty(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  bonds:
    bn0:
      dhcp4: true''')

        self.assert_nm({'bn0': '''[connection]
id=netplan-bn0
type=bond
interface-name=bn0

[ipv4]
method=auto

[ipv6]
method=ignore
'''})

    def test_bond_components(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    eno1: {}
    switchport:
      match:
        name: enp2s1
  bonds:
    bn0:
      interfaces: [eno1, switchport]
      dhcp4: true''')

        self.assert_nm({'eno1': '''[connection]
id=netplan-eno1
type=ethernet
interface-name=eno1
slave-type=bond # wokeignore:rule=slave
master=bn0 # wokeignore:rule=master

[ethernet]
wake-on-lan=0

[ipv4]
method=link-local

[ipv6]
method=ignore
''',
                        'switchport': '''[connection]
id=netplan-switchport
type=ethernet
interface-name=enp2s1
slave-type=bond # wokeignore:rule=slave
master=bn0 # wokeignore:rule=master

[ethernet]
wake-on-lan=0

[ipv4]
method=link-local

[ipv6]
method=ignore
''',
                        'bn0': '''[connection]
id=netplan-bn0
type=bond
interface-name=bn0

[ipv4]
method=auto

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'eno1' + NM_MANAGED % 'enp2s1' + NM_MANAGED % 'bn0')

    def test_bond_empty_params(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    eno1: {}
    switchport:
      match:
        name: enp2s1
  bonds:
    bn0:
      interfaces: [eno1, switchport]
      parameters: {}
      dhcp4: true''')

        self.assert_nm({'eno1': '''[connection]
id=netplan-eno1
type=ethernet
interface-name=eno1
slave-type=bond # wokeignore:rule=slave
master=bn0 # wokeignore:rule=master

[ethernet]
wake-on-lan=0

[ipv4]
method=link-local

[ipv6]
method=ignore
''',
                        'switchport': '''[connection]
id=netplan-switchport
type=ethernet
interface-name=enp2s1
slave-type=bond # wokeignore:rule=slave
master=bn0 # wokeignore:rule=master

[ethernet]
wake-on-lan=0

[ipv4]
method=link-local

[ipv6]
method=ignore
''',
                        'bn0': '''[connection]
id=netplan-bn0
type=bond
interface-name=bn0

[ipv4]
method=auto

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'eno1' + NM_MANAGED % 'enp2s1' + NM_MANAGED % 'bn0')

    def test_bond_with_params_all_members_active(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    eno1: {}
    switchport:
      match:
        name: enp2s1
  bonds:
    bn0:
      interfaces: [eno1, switchport]
      parameters:
        mode: 802.3ad
        lacp-rate: slow
        mii-monitor-interval: 10
        min-links: 10
        up-delay: 10
        down-delay: 10
        all-members-active: true
        transmit-hash-policy: none
        ad-select: none
        arp-interval: 10
        arp-validate: all
        arp-all-targets: all
        arp-ip-targets:
          - 10.10.10.10
          - 20.20.20.20
        fail-over-mac-policy: none
        gratuitious-arp: 10
        packets-per-member: 10
        primary-reselect-policy: none
        resend-igmp: 10
        learn-packet-interval: 10
      dhcp4: true''')

        self.assert_nm({'eno1': '''[connection]
id=netplan-eno1
type=ethernet
interface-name=eno1
slave-type=bond # wokeignore:rule=slave
master=bn0 # wokeignore:rule=master

[ethernet]
wake-on-lan=0

[ipv4]
method=link-local

[ipv6]
method=ignore
''',
                        'switchport': '''[connection]
id=netplan-switchport
type=ethernet
interface-name=enp2s1
slave-type=bond # wokeignore:rule=slave
master=bn0 # wokeignore:rule=master

[ethernet]
wake-on-lan=0

[ipv4]
method=link-local

[ipv6]
method=ignore
''',
                        'bn0': '''[connection]
id=netplan-bn0
type=bond
interface-name=bn0

[bond]
mode=802.3ad
lacp_rate=slow
miimon=10
min_links=10
xmit_hash_policy=none
ad_select=none
all_slaves_active=1 # wokeignore:rule=slave
arp_interval=10
arp_ip_target=10.10.10.10,20.20.20.20
arp_validate=all
arp_all_targets=all
updelay=10
downdelay=10
fail_over_mac=none
num_grat_arp=10
num_unsol_na=10
packets_per_slave=10 # wokeignore:rule=slave
primary_reselect=none
resend_igmp=10
lp_interval=10

[ipv4]
method=auto

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'eno1' + NM_MANAGED % 'enp2s1' + NM_MANAGED % 'bn0')

    def test_bond_with_params(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    eno1: {}
    switchport:
      match:
        name: enp2s1
  bonds:
    bn0:
      interfaces: [eno1, switchport]
      parameters:
        mode: 802.3ad
        lacp-rate: slow
        mii-monitor-interval: 10
        min-links: 10
        up-delay: 10
        down-delay: 10
        all-slaves-active: true # wokeignore:rule=slave
        transmit-hash-policy: none
        ad-select: none
        arp-interval: 10
        arp-validate: all
        arp-all-targets: all
        arp-ip-targets:
          - 10.10.10.10
          - 20.20.20.20
        fail-over-mac-policy: none
        gratuitious-arp: 10
        packets-per-slave: 10 # wokeignore:rule=slave
        primary-reselect-policy: none
        resend-igmp: 10
        learn-packet-interval: 10
      dhcp4: true''', skip_generated_yaml_validation=True)
        # Skipping the yaml validation above because the emitter will use
        # all-members-active and packets-per-member by default.

        self.assert_nm({'eno1': '''[connection]
id=netplan-eno1
type=ethernet
interface-name=eno1
slave-type=bond # wokeignore:rule=slave
master=bn0 # wokeignore:rule=master

[ethernet]
wake-on-lan=0

[ipv4]
method=link-local

[ipv6]
method=ignore
''',
                        'switchport': '''[connection]
id=netplan-switchport
type=ethernet
interface-name=enp2s1
slave-type=bond # wokeignore:rule=slave
master=bn0 # wokeignore:rule=master

[ethernet]
wake-on-lan=0

[ipv4]
method=link-local

[ipv6]
method=ignore
''',
                        'bn0': '''[connection]
id=netplan-bn0
type=bond
interface-name=bn0

[bond]
mode=802.3ad
lacp_rate=slow
miimon=10
min_links=10
xmit_hash_policy=none
ad_select=none
all_slaves_active=1 # wokeignore:rule=slave
arp_interval=10
arp_ip_target=10.10.10.10,20.20.20.20
arp_validate=all
arp_all_targets=all
updelay=10
downdelay=10
fail_over_mac=none
num_grat_arp=10
num_unsol_na=10
packets_per_slave=10 # wokeignore:rule=slave
primary_reselect=none
resend_igmp=10
lp_interval=10

[ipv4]
method=auto

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'eno1' + NM_MANAGED % 'enp2s1' + NM_MANAGED % 'bn0')

    def test_bond_primary_member(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    eno1: {}
    switchport:
      match:
        name: enp2s1
  bonds:
    bn0:
      interfaces: [eno1, switchport]
      parameters:
        mode: active-backup
        primary: eno1
      dhcp4: true''')

        self.assert_nm({'eno1': '''[connection]
id=netplan-eno1
type=ethernet
interface-name=eno1
slave-type=bond # wokeignore:rule=slave
master=bn0 # wokeignore:rule=master

[ethernet]
wake-on-lan=0

[ipv4]
method=link-local

[ipv6]
method=ignore
''',
                        'switchport': '''[connection]
id=netplan-switchport
type=ethernet
interface-name=enp2s1
slave-type=bond # wokeignore:rule=slave
master=bn0 # wokeignore:rule=master

[ethernet]
wake-on-lan=0

[ipv4]
method=link-local

[ipv6]
method=ignore
''',
                        'bn0': '''[connection]
id=netplan-bn0
type=bond
interface-name=bn0

[bond]
mode=active-backup
primary=eno1

[ipv4]
method=auto

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'eno1' + NM_MANAGED % 'enp2s1' + NM_MANAGED % 'bn0')


class TestConfigErrors(TestBase):

    def test_bond_invalid_mode(self):
        err = self.generate('''network:
  version: 2
  ethernets:
    eno1:
      match:
        name: eth0
  bonds:
    bond0:
      interfaces: [eno1]
      parameters:
        mode: lacp
        arp-ip-targets:
          - 2001:dead:beef::1
      dhcp4: true''', expect_fail=True)
        self.assertIn("unknown bond mode 'lacp'", err)

    def test_bond_invalid_lacp_rate(self):
        err = self.generate('''network:
  version: 2
  ethernets:
    eno1:
      match:
        name: eth0
  bonds:
    bond0:
      interfaces: [eno1]
      parameters:
        lacp-rate: abcd
      dhcp4: true''', expect_fail=True)
        self.assertIn("unknown lacp-rate value 'abcd'", err)

    def test_bond_invalid_arp_target(self):
        self.generate('''network:
  version: 2
  ethernets:
    eno1:
      match:
        name: eth0
  bonds:
    bond0:
      interfaces: [eno1]
      parameters:
        arp-ip-targets:
          - 2001:dead:beef::1
      dhcp4: true''', expect_fail=True)

    def test_bond_invalid_primary_member(self):
        self.generate('''network:
  version: 2
  ethernets:
    eno1:
      match:
        name: eth0
  bonds:
    bond0:
      interfaces: [eno1]
      parameters:
        primary: wigglewiggle
      dhcp4: true''', expect_fail=True)

    def test_bond_duplicate_primary_member(self):
        self.generate('''network:
  version: 2
  ethernets:
    eno1:
      match:
        name: eth0
    eno2:
      match:
        name: eth1
  bonds:
    bond0:
      interfaces: [eno1, eno2]
      parameters:
        primary: eno1
        primary: eno2
      dhcp4: true''', expect_fail=True)

    def test_bond_multiple_assignments(self):
        err = self.generate('''network:
  version: 2
  ethernets:
    eno1: {}
  bonds:
    bond0:
      interfaces: [eno1]
    bond1:
      interfaces: [eno1]''', expect_fail=True)
        self.assertIn("bond1: interface 'eno1' is already assigned to bond bond0", err)

    def test_bond_primary_multiple_assignments(self):
        err = self.generate('''network:
  version: 2
  ethernets:
    eno1: {}
  bonds:
    bond0:
      parameters:
        primary: eno1
    bond1:
      parameters:
        primary: eno1''', expect_fail=True)
        self.assertIn("bond1: interface 'eno1' is already a primary of bond0", err)

    def test_bond_bridge_cross_assignments1(self):
        err = self.generate('''network:
  version: 2
  ethernets:
    eno1: {}
  bonds:
    bond0:
      interfaces: [eno1]
  bridges:
    br1:
      interfaces: [eno1]''', expect_fail=True)
        self.assertIn("br1: interface 'eno1' is already assigned to bond bond0", err)

    def test_bond_bridge_cross_assignments2(self):
        err = self.generate('''network:
  version: 2
  ethernets:
    eno1: {}
  bridges:
    br0:
      interfaces: [eno1]
  bonds:
    bond1:
      interfaces: [eno1]''', expect_fail=True)
        self.assertIn("bond1: interface 'eno1' is already assigned to bridge br0", err)

    def test_bond_bridge_nested_assignments(self):
        self.generate('''network:
  version: 2
  ethernets:
    eno1: {}
  bonds:
    bond0:
      interfaces: [eno1]
  bridges:
    br1:
      interfaces: [bond0]''')



#
# Tests for DHCP override handling in netplan generator
#
# Copyright (C) 2018 Canonical, Ltd.
# Author: Mathieu Trudel-Lapierre <mathieu.trudel.lapierre@canonical.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .base import (TestBase, ND_DHCP4, ND_DHCP4_NOMTU, ND_DHCP6,
                   ND_DHCP6_NOMTU, ND_DHCPYES, ND_DHCPYES_NOMTU)


class TestNetworkd(TestBase):

    # Common tests for dhcp override booleans
    def assert_dhcp_overrides_bool(self, override_name, networkd_name):
        # dhcp4 yes
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: yes
''' % override_name)
        # silently ignored since yes is the default
        self.assert_networkd({'engreen.network': ND_DHCP4 % 'engreen'})

        # dhcp6 yes
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp6: yes
      dhcp6-overrides:
        %s: yes
''' % override_name)
        # silently ignored since yes is the default
        self.assert_networkd({'engreen.network': ND_DHCP6 % 'engreen'})

        # dhcp4 and dhcp6 both yes
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: yes
      dhcp6: yes
      dhcp6-overrides:
        %s: yes
''' % (override_name, override_name))
        # silently ignored since yes is the default
        self.assert_networkd({'engreen.network': ND_DHCPYES % 'engreen'})

        # dhcp4 no
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: no
''' % override_name)
        self.assert_networkd({'engreen.network': ND_DHCP4 % 'engreen' + '%s=false\n' % networkd_name})

        # dhcp6 no
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp6: yes
      dhcp6-overrides:
        %s: no
''' % override_name)
        self.assert_networkd({'engreen.network': ND_DHCP6 % 'engreen' + '%s=false\n' % networkd_name})

        # dhcp4 and dhcp6 both no
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: no
      dhcp6: yes
      dhcp6-overrides:
        %s: no
''' % (override_name, override_name))
        self.assert_networkd({'engreen.network': ND_DHCPYES % 'engreen' + '%s=false\n' % networkd_name})

        # mismatched values
        err = self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: yes
      dhcp6: yes
      dhcp6-overrides:
        %s: no
''' % (override_name, override_name), expect_fail=True)
        self.assertIn('ERROR: engreen: networkd requires that '
                      '%s has the same value in both dhcp4_overrides and dhcp6_overrides' % override_name,
                      err.strip())

    # Common tests for dhcp override strings
    def assert_dhcp_overrides_string(self, override_name, networkd_name):
        # dhcp4 only
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: foo
''' % override_name)
        self.assert_networkd({'engreen.network': ND_DHCP4 % 'engreen' + '%s=foo\n' % networkd_name})

        # dhcp6 only
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp6: yes
      dhcp6-overrides:
        %s: foo
''' % override_name)
        self.assert_networkd({'engreen.network': ND_DHCP6 % 'engreen' + '%s=foo\n' % networkd_name})

        # dhcp4 and dhcp6
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: foo
      dhcp6: yes
      dhcp6-overrides:
        %s: foo
''' % (override_name, override_name))
        self.assert_networkd({'engreen.network': ND_DHCPYES % 'engreen' + '%s=foo\n' % networkd_name})

        # mismatched values
        err = self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: foo
      dhcp6: yes
      dhcp6-overrides:
        %s: bar
''' % (override_name, override_name), expect_fail=True)
        self.assertIn('ERROR: engreen: networkd requires that '
                      '%s has the same value in both dhcp4_overrides and dhcp6_overrides' % override_name,
                      err.strip())

    # Common tests for dhcp override booleans
    def assert_dhcp_mtu_overrides_bool(self, override_name, networkd_name):
        # dhcp4 yes
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: yes
''' % override_name)
        self.assert_networkd({'engreen.network': ND_DHCP4 % 'engreen'})

        # dhcp6 yes
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp6: yes
      dhcp6-overrides:
        %s: yes
''' % override_name)
        # silently ignored since yes is the default
        self.assert_networkd({'engreen.network': ND_DHCP6 % 'engreen'})

        # dhcp4 and dhcp6 both yes
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: yes
      dhcp6: yes
      dhcp6-overrides:
        %s: yes
''' % (override_name, override_name))
        # silently ignored since yes is the default
        self.assert_networkd({'engreen.network': ND_DHCPYES % 'engreen'})

        # dhcp4 no
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: no
''' % override_name)
        self.assert_networkd({'engreen.network': ND_DHCP4_NOMTU % 'engreen'})

        # dhcp6 no
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp6: yes
      dhcp6-overrides:
        %s: no
''' % override_name)
        self.assert_networkd({'engreen.network': ND_DHCP6_NOMTU % 'engreen'})

        # dhcp4 and dhcp6 both no
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: no
      dhcp6: yes
      dhcp6-overrides:
        %s: no
''' % (override_name, override_name))
        self.assert_networkd({'engreen.network': ND_DHCPYES_NOMTU % 'engreen'})

        # mismatched values
        err = self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: yes
      dhcp6: yes
      dhcp6-overrides:
        %s: no
''' % (override_name, override_name), expect_fail=True)
        self.assertIn('ERROR: engreen: networkd requires that '
                      '%s has the same value in both dhcp4_overrides and dhcp6_overrides' % override_name,
                      err.strip())

    def assert_dhcp_overrides_guint(self, override_name, networkd_name):
        # dhcp4 only
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: 6000
''' % override_name)
        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
DHCP=ipv4
LinkLocalAddressing=ipv6

[DHCP]
RouteMetric=6000
UseMTU=true
'''})

        # dhcp6 only
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp6: yes
      dhcp6-overrides:
        %s: 6000
''' % override_name)
        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
DHCP=ipv6
LinkLocalAddressing=ipv6

[DHCP]
RouteMetric=6000
UseMTU=true
'''})

        # dhcp4 and dhcp6
        self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: 6000
      dhcp6: yes
      dhcp6-overrides:
        %s: 6000
''' % (override_name, override_name))
        self.assert_networkd({'engreen.network': '''[Match]
Name=engreen

[Network]
DHCP=yes
LinkLocalAddressing=ipv6

[DHCP]
RouteMetric=6000
UseMTU=true
'''})

        # mismatched values
        err = self.generate('''network:
  version: 2
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        %s: 3333
      dhcp6: yes
      dhcp6-overrides:
        %s: 5555
''' % (override_name, override_name), expect_fail=True)
        self.assertIn('ERROR: engreen: networkd requires that '
                      '%s has the same value in both dhcp4_overrides and dhcp6_overrides' % override_name,
                      err.strip())

    def test_dhcp_overrides_use_dns(self):
        self.assert_dhcp_overrides_bool('use-dns', 'UseDNS')

    def test_dhcp_overrides_use_domains(self):
        self.assert_dhcp_overrides_string('use-domains', 'UseDomains')

    def test_dhcp_overrides_use_ntp(self):
        self.assert_dhcp_overrides_bool('use-ntp', 'UseNTP')

    def test_dhcp_overrides_send_hostname(self):
        self.assert_dhcp_overrides_bool('send-hostname', 'SendHostname')

    def test_dhcp_overrides_use_hostname(self):
        self.assert_dhcp_overrides_bool('use-hostname', 'UseHostname')

    def test_dhcp_overrides_hostname(self):
        self.assert_dhcp_overrides_string('hostname', 'Hostname')

    def test_dhcp_overrides_use_mtu(self):
        self.assert_dhcp_mtu_overrides_bool('use-mtu', 'UseMTU')

    def test_dhcp_overrides_default_metric(self):
        self.assert_dhcp_overrides_guint('route-metric', 'RouteMetric')

    def test_dhcp_overrides_use_routes(self):
        self.assert_dhcp_overrides_bool('use-routes', 'UseRoutes')


class TestNetworkManager(TestBase):

    def test_override_default_metric_v4(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      dhcp4: yes
      dhcp4-overrides:
        route-metric: 3333
''')
        # silently ignored since yes is the default
        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=auto
route-metric=3333

[ipv6]
method=ignore
'''})

    def test_override_default_metric_v6(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  ethernets:
    engreen:
      dhcp4: yes
      dhcp6: yes
      dhcp6-overrides:
        route-metric: 6666
''')
        # silently ignored since yes is the default
        self.assert_nm({'engreen': '''[connection]
id=netplan-engreen
type=ethernet
interface-name=engreen

[ethernet]
wake-on-lan=0

[ipv4]
method=auto

[ipv6]
method=auto
ip6-privacy=0
route-metric=6666
'''})



#!/usr/bin/python3
#
# Test runner for netplan integration tests.
#
# These need to be run in a VM and do change the system
# configuration.
#
# Copyright (C) 2018 Canonical, Ltd.
# Author: Mathieu Trudel-Lapierre <mathieu.trudel-lapierre@canonical.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import glob
import os
import subprocess
import textwrap
import sys

tests_dir = os.path.dirname(os.path.abspath(__file__))

default_backends = ['networkd', 'NetworkManager']
fixtures = ["__init__.py", "base.py", "run.py"]

possible_tests = []
testfiles = glob.glob(os.path.join(tests_dir, "*.py"))
for pyfile in testfiles:
    filename = os.path.basename(pyfile)
    if filename not in fixtures:
        possible_tests.append(filename.split('.')[0])


def dedupe(duped_list):
    deduped = set()
    for item in duped_list:
        real_items = item.split(",")
        for real_item in real_items:
            deduped.add(real_item)
    return deduped


# XXX: omg, this is ugly :)
parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                 description=textwrap.dedent("""
Test runner for netplan integration tests

Available tests:
{}
""".format("\n".join("    - {}".format(x) for x in sorted(possible_tests)))))

parser.add_argument('--test', action='append', help="List of tests to be run")
parser.add_argument('--backend', action='append', help="List of backends to test (NetworkManager, networkd)")

args = parser.parse_args()

requested_tests = set()
backends = set()

if args.test is not None:
    requested_tests = dedupe(args.test)
else:
    requested_tests.update(possible_tests)

if args.backend is not None:
    backends = dedupe(args.backend)
else:
    backends.update(default_backends)

os.environ["NETPLAN_TEST_BACKENDS"] = ",".join(backends)

run_with_ignore_errors = os.environ.get("NETPLAN_PARSER_IGNORE_ERRORS", "0")

returncode = 0
for test in requested_tests:
    os.environ["NETPLAN_PARSER_IGNORE_ERRORS"] = "0"
    ret = subprocess.call(['python3', os.path.join(tests_dir, "{}.py".format(test))])
    if returncode == 0 and ret != 0:
        returncode = ret

    # Running tests again with NETPLAN_PARSER_IGNORE_ERRORS
    # If NETPLAN_PARSER_IGNORE_ERRORS=0 is defined initially, this step will be skipped
    # When this variable is set to 1, the netplan generator will set the IGNORE_ERRORS flag
    if run_with_ignore_errors == "1":
        print(f"Running '{test}' tests again with IGNORE_ERRORS flag set", flush=True)
        os.environ["NETPLAN_PARSER_IGNORE_ERRORS"] = "1"
        ret = subprocess.call(['python3', os.path.join(tests_dir, "{}.py".format(test))])
        if returncode == 0 and ret != 0:
            returncode = ret

sys.exit(returncode)



import sys
import datetime
import atexit
import fileinput
import os
import re

# Custom configuration for the Sphinx documentation builder.
# All configuration specific to your project should be done in this file.
#
# The file is included in the common conf.py configuration file.
# You can modify any of the settings below or add any configuration that
# is not covered by the common conf.py file.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/
#
# If you're not familiar with Sphinx and don't want to use advanced
# features, it is sufficient to update the settings in the "Project
# information" section.

############################################################
# Project information
############################################################

# Product name
project = 'Netplan'
author = 'Netplan team'

# The title you want to display for the documentation in the sidebar.
# You might want to include a version number here.
# To not display any title, set this option to an empty string.
html_title = project + ' documentation'

# The default value uses the current year as the copyright year.
#
# For static works, it is common to provide the year of first publication.
# Another option is to give the first year and the current year
# for documentation that is often changed, e.g. 2022–2023 (note the en-dash).
#
# A way to check a GitHub repo's creation date is to obtain a classic GitHub
# token with 'repo' permissions here: https://github.com/settings/tokens
# Next, use 'curl' and 'jq' to extract the date from the GitHub API's output:
#
# curl -H 'Authorization: token <TOKEN>' \
#   -H 'Accept: application/vnd.github.v3.raw' \
#   https://api.github.com/repos/canonical/<REPO> | jq '.created_at'

copyright = '%s, %s' % (datetime.date.today().year, author)

# Open Graph configuration - defines what is displayed as a link preview
# when linking to the documentation from another website (see https://ogp.me/)
# The URL where the documentation will be hosted (leave empty if you
# don't know yet)
ogp_site_url = 'https://netplan.readthedocs.io/en/stable/netplan-apply/'
# The documentation website name (usually the same as the product name)
ogp_site_name = project
# The URL of an image or logo that is used in the preview
ogp_image = 'https://assets.ubuntu.com/v1/253da317-image-document-ubuntudocs.svg'

# Update with the local path to the favicon for your product
# (default is the circle of friends)
html_favicon = '.sphinx/_static/favicon.png'

# The logo
html_logo = '.sphinx/_static/netplan.svg'

# (Some settings must be part of the html_context dictionary, while others
#  are on root level. Don't move the settings.)
html_context = {

    # Change to the link to the website of your product (without "https://")
    # For example: "ubuntu.com/lxd" or "microcloud.is"
    # If there is no product website, edit the header template to remove the
    # link (see the readme for instructions).
    'product_page': 'netplan.io',

    # Add your product tag (the orange part of your logo, will be used in the
    # header) to ".sphinx/_static" and change the path here (start with "_static")
    # (default is the circle of friends)
    'product_tag': '_static/tag.png',

    # Change to the discourse instance you want to be able to link to
    # using the :discourse: metadata at the top of a file
    # (use an empty value if you don't want to link)
    'discourse': 'https://discourse.ubuntu.com/c/foundations/',

    # Change to the Mattermost channel you want to link to
    # (use an empty value if you don't want to link)
    'mattermost': 'https://chat.canonical.com/canonical/channels/documentation',

    # Change to the GitHub URL for your project
    'github_url': 'https://github.com/canonical/netplan',

    # Change to the branch for this version of the documentation
    'github_version': 'main',

    # Change to the folder that contains the documentation
    # (usually "/" or "/docs/")
    'github_folder': '/doc/',

    # Change to an empty value if your GitHub repo doesn't have issues enabled.
    # This will disable the feedback button and the issue link in the footer.
    'github_issues': 'https://bugs.launchpad.net/netplan/',

    # Controls the existence of Previous / Next buttons at the bottom of pages
    # Valid options: none, prev, next, both
    'sequential_nav': "none"
}

# If your project is on documentation.ubuntu.com, specify the project
# slug (for example, "lxd") here.
slug = ""

############################################################
# Redirects
############################################################

# Set up redirects (https://documatt.gitlab.io/sphinx-reredirects/usage.html)
# For example: 'explanation/old-name.html': '../how-to/prettify.html',

redirects = {
    'README.md': '/',
    'netplan': '/netplan-yaml',
}

############################################################
# Link checker exceptions
############################################################

# Links to ignore when checking links

linkcheck_ignore = [
    'http://127.0.0.1:8000'
]

# Pages on which to ignore anchors
# (This list will be appended to linkcheck_anchors_ignore_for_url)

custom_linkcheck_anchors_ignore_for_url = []

############################################################
# Additions to default configuration
############################################################

# The following settings are appended to the default configuration.
# Use them to extend the default functionality.

# Add extensions
custom_extensions = [
    'breathe',
]

# Add MyST extensions
custom_myst_extensions = ["colon_fence"]

# Add files or directories that should be excluded from processing.
custom_excludes = [
    'doc-cheat-sheet*',
    'manpage-footer.md',
    'manpage-header.md',
]

# Add CSS files (located in .sphinx/_static/)
custom_html_css_files = []

# Add JavaScript files (located in .sphinx/_static/)
custom_html_js_files = []

# The following settings override the default configuration.

# Specify a reST string that is included at the end of each file.
# If commented out or empty, use the default (which pulls the reuse/links.txt
# file into each reST file).
custom_rst_epilog = ''

# By default, the documentation includes a feedback button at the top.
# You can disable it by setting the following configuration to True.
disable_feedback_button = False

# Add tags that you want to use for conditional inclusion of text
# (https://www.sphinx-doc.org/)
custom_tags = []

############################################################
# Additional configuration
############################################################

# Add any configuration that is not covered by the common conf.py file.

smartquotes_action = 'qe'

# Doxygen
# https://breathe.readthedocs.io/en/latest/directives.html
# breathe_projects = {"Netplan": "../doxyxml/"}
breathe_projects_source = {"auto-apidoc": ("../", [
    "include/netdef.h",
    "include/netplan.h",
    "include/parse-nm.h",
    "include/parse.h",
    "include/state.h",
    "include/types.h",
    "include/util.h",
    # "src/error.c",
    # "src/names.c",
    # "src/netplan.c",
    # "src/parse-nm.c",
    # "src/parse.c",
    # "src/types.c",
    # "src/util.c",
    # "src/validation.c",
])}

breathe_doxygen_config_options = {
    'MACRO_EXPANSION': 'YES',
    'EXPAND_ONLY_PREDEF': 'YES',
    'PREDEFINED': 'NETPLAN_PUBLIC NETPLAN_DEPRECATED',
}
breathe_domain_by_extension = {
    "h": "c",
    "c": "c",
}
# breathe_doxygen_aliases =
breathe_default_project = "auto-apidoc"

# Options for MyST
myst_title_to_header = True
suppress_warnings = ['myst.xref_missing']

#
# sys.path.append('./')
#
# from custom_conf import *

# Configuration file for the Sphinx documentation builder.
# You should not do any modifications to this file. Put your custom
# configuration into the custom_conf.py file.
# If you need to change this file, contribute the changes upstream.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/

############################################################
# Extensions
############################################################

extensions = [
    'sphinx_design',
    'sphinx_tabs.tabs',
    'sphinx_reredirects',
    'youtube-links',
    'related-links',
    'custom-rst-roles',
    'terminal-output',
    'sphinx_copybutton',
    'sphinxext.opengraph',
    'myst_parser',
    'sphinxcontrib.jquery',
    'notfound.extension'
]
extensions.extend(custom_extensions)

# Configuration for extensions

# Additional MyST syntax
myst_enable_extensions = [
    'substitution',
    'deflist',
    'linkify'
]
myst_enable_extensions.extend(custom_myst_extensions)

# Used for related links
if 'discourse_prefix' not in html_context and 'discourse' not in html_context:
    html_context['discourse_prefix'] = html_context['discourse'] + '/t/'

# The default for notfound_urls_prefix usually works, but not for
# documentation on documentation.ubuntu.com
if slug:
    notfound_urls_prefix = '/' + slug + '/en/latest/'

notfound_context = {
    'title': 'Page not found',
    'body': '<h1>Page not found</h1>\n\n<p>Sorry, but the documentation page \
        that you are looking for was not found.</p>\n<p>Documentation changes \
        over time, and pages are moved around. We try to redirect you to the \
        updated content where possible, but unfortunately, that didn\'t work \
        this time (maybe because the content you were looking for does not \
        exist in this version of the documentation).</p>\n<p>You can try to use \
        the navigation to locate the content you\'re looking for, or search for \
        a similar page.</p>\n',
}

# Default image for OGP (to prevent font errors, see
# https://github.com/canonical/sphinx-docs-starter-pack/pull/54 )
if 'ogp_image' not in locals():
    ogp_image = 'https://assets.ubuntu.com/v1/253da317-image-document-ubuntudocs.svg'

############################################################
# General configuration
############################################################

exclude_patterns = [
    '_build',
    'Thumbs.db',
    '.DS_Store',
    '.sphinx',
    'reuse',
]
exclude_patterns.extend(custom_excludes)

rst_epilog = '''
.. include:: /reuse/links.txt
'''

if 'custom_rst_epilog' in locals() and 'custom_rst_epilog' != '':
    rst_epilog = custom_rst_epilog

source_suffix = {
    '.rst': 'restructuredtext',
    '.md': 'markdown',
}

if 'conf_py_path' not in html_context and 'github_folder' not in html_context:
    html_context['conf_py_path'] = html_context['github_folder']

# For ignoring specific links
linkcheck_anchors_ignore_for_url = [
    r'https://github\.com/.*'
]
linkcheck_anchors_ignore_for_url.extend(custom_linkcheck_anchors_ignore_for_url)

# Tags cannot be added directly in custom_conf.py, so add them here

tags = ()

for tag in custom_tags:
    tags.add(tag)

############################################################
# Styling
############################################################

# Find the current builder
builder = 'dirhtml'
if '-b' in sys.argv:
    builder = sys.argv[sys.argv.index('-b')+1]

# Setting templates_path for epub makes the build fail
if builder == 'dirhtml' or builder == 'html':
    templates_path = ['.sphinx/_templates']

# Theme configuration
html_theme = 'furo'
html_last_updated_fmt = ''
html_permalinks_icon = '¶'

if html_title == '':
    html_theme_options = {
        'sidebar_hide_name': True
        }

############################################################
# Additional files
############################################################

html_static_path = ['.sphinx/_static']

html_css_files = [
    'custom.css',
    'header.css',
    'github_issue_links.css',
    'furo_colors.css'
]
html_css_files.extend(custom_html_css_files)

html_js_files = ['header-nav.js']
if 'github_issues' in html_context and html_context['github_issues'] and not disable_feedback_button:
    html_js_files.append('github_issue_links.js')
html_js_files.extend(custom_html_js_files)


# Clip the "How to" string from how-to titles in the left navigation sidebar
#     and capitalizes the first letter of the remaining title.
def clip_howto():
    pattern = re.compile(r'(toctree-l2.*>)How to ([a-zA-Z])')
    # Check if the build is running on ReadTheDocs
    if os.environ.get('READTHEDOCS') == 'True':
        output_dir = os.environ.get('READTHEDOCS_OUTPUT') + '/html'
    else:
        output_dir = '_build'

    print("Clipping 'How to' from the Table of Contents...")

    for root, dirs, files in os.walk(output_dir):
        for file in files:
            if file.endswith('.html'):
                file_path = os.path.join(root, file)

                with fileinput.FileInput(file_path, inplace=True) as f:
                    for line in f:
                        line = pattern.sub(lambda match: match.group(1) + match.group(2).upper(), line)
                        print(line, end='')


# Register the clip_howto function to be called on exit
atexit.register(clip_howto)
# End how-to clipping



#
# Tests for gsm/cdma modem devices config generated via netplan
#
# Copyright (C) 2020 Canonical, Ltd.
# Author: Lukas Märdian <lukas.maerdian@canonical.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .base import TestBase, NM_MANAGED


class TestNetworkd(TestBase):
    '''networkd output'''

    def test_not_supported(self):
        # does not produce any output, but fails with:
        # "networkd backend does not support GSM modem configuration"
        err = self.generate('''network:
  version: 2
  modems:
    mobilephone:
      auto-config: true''', expect_fail=True)
        self.assertIn("ERROR: mobilephone: networkd backend does not support GSM/CDMA modem configuration", err)

        self.assert_networkd({})
        self.assert_nm({})


class TestNetworkManager(TestBase):
    '''networkmanager output'''

    def test_cdma_config(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  modems:
    mobilephone:
      mtu: 1542
      number: "#666"
      username: test-user
      password: s0s3kr1t''')
        self.assert_nm({'mobilephone': '''[connection]
id=netplan-mobilephone
type=cdma
interface-name=mobilephone

[cdma]
password=s0s3kr1t
username=test-user
mtu=1542
number=#666

[ipv4]
method=link-local

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'mobilephone')

    def test_gsm_auto_config(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  modems:
    mobilephone:
      auto-config: true''')
        self.assert_nm({'mobilephone': '''[connection]
id=netplan-mobilephone
type=gsm
interface-name=mobilephone

[gsm]
auto-config=true

[ipv4]
method=link-local

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'mobilephone')

    def test_gsm_auto_config_implicit(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  modems:
    mobilephone:
      number: "*99#"
      mtu: 1600
      pin: "1234"''')
        self.assert_nm({'mobilephone': '''[connection]
id=netplan-mobilephone
type=gsm
interface-name=mobilephone

[gsm]
auto-config=true
mtu=1600
number=*99#
pin=1234

[ipv4]
method=link-local

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'mobilephone')

    def test_gsm_apn(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  modems:
    mobilephone:
      apn: internet''')
        self.assert_nm({'mobilephone': '''[connection]
id=netplan-mobilephone
type=gsm
interface-name=mobilephone

[gsm]
apn=internet

[ipv4]
method=link-local

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'mobilephone')

    def test_gsm_apn_username_password(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  modems:
    mobilephone:
      apn: internet
      username: some-user
      password: some-pass''')
        self.assert_nm({'mobilephone': '''[connection]
id=netplan-mobilephone
type=gsm
interface-name=mobilephone

[gsm]
apn=internet
password=some-pass
username=some-user

[ipv4]
method=link-local

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'mobilephone')

    def test_gsm_device_id(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  modems:
    mobilephone:
      device-id: test''')
        self.assert_nm({'mobilephone': '''[connection]
id=netplan-mobilephone
type=gsm
interface-name=mobilephone

[gsm]
auto-config=true
device-id=test

[ipv4]
method=link-local

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'mobilephone')

    def test_gsm_network_id(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  modems:
    mobilephone:
      network-id: test''')
        self.assert_nm({'mobilephone': '''[connection]
id=netplan-mobilephone
type=gsm
interface-name=mobilephone

[gsm]
auto-config=true
network-id=test

[ipv4]
method=link-local

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'mobilephone')

    def test_gsm_pin(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  modems:
    mobilephone:
      pin: 1234''')
        self.assert_nm({'mobilephone': '''[connection]
id=netplan-mobilephone
type=gsm
interface-name=mobilephone

[gsm]
auto-config=true
pin=1234

[ipv4]
method=link-local

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'mobilephone')

    def test_gsm_sim_id(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  modems:
    mobilephone:
      sim-id: test''')
        self.assert_nm({'mobilephone': '''[connection]
id=netplan-mobilephone
type=gsm
interface-name=mobilephone

[gsm]
auto-config=true
sim-id=test

[ipv4]
method=link-local

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'mobilephone')

    def test_gsm_sim_operator_id(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  modems:
    mobilephone:
      sim-operator-id: test''')
        self.assert_nm({'mobilephone': '''[connection]
id=netplan-mobilephone
type=gsm
interface-name=mobilephone

[gsm]
auto-config=true
sim-operator-id=test

[ipv4]
method=link-local

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'mobilephone')

    def test_gsm_example(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  modems:
    cdc-wdm1:
      mtu: 1600
      apn: ISP.CINGULAR
      username: ISP@CINGULARGPRS.COM
      password: CINGULAR1
      number: "*99#"
      network-id: 24005
      device-id: da812de91eec16620b06cd0ca5cbc7ea25245222
      pin: 2345
      sim-id: 89148000000060671234
      sim-operator-id: 310260''')
        self.assert_nm({'cdc-wdm1': '''[connection]
id=netplan-cdc-wdm1
type=gsm
interface-name=cdc-wdm1

[gsm]
apn=ISP.CINGULAR
password=CINGULAR1
username=ISP@CINGULARGPRS.COM
device-id=da812de91eec16620b06cd0ca5cbc7ea25245222
mtu=1600
network-id=24005
number=*99#
pin=2345
sim-id=89148000000060671234
sim-operator-id=310260

[ipv4]
method=link-local

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'cdc-wdm1')

    def test_modem_nm_integration(self):
        self.generate('''network:
  version: 2
  renderer: NetworkManager
  modems:
    mobilephone:
      auto-config: true
      networkmanager:
        uuid: b22d8f0f-3f34-46bd-ac28-801fa87f1eb6''')
        self.assert_nm({'mobilephone': '''[connection]
id=netplan-mobilephone
type=gsm
uuid=b22d8f0f-3f34-46bd-ac28-801fa87f1eb6
interface-name=mobilephone

[gsm]
auto-config=true

[ipv4]
method=link-local

[ipv6]
method=ignore
'''})
        self.assert_networkd({})
        self.assert_nm_udev(NM_MANAGED % 'mobilephone')

    def test_modem_nm_integration_gsm_cdma(self):
        self.generate('''network:
  version: 2
  modems:
    NM-a08c5805-7cf5-43f7-afb9-12cb30f6eca3:
      renderer: NetworkManager
      match: {}
      apn: internet2.voicestream.com
      networkmanager:
        uuid: a08c5805-7cf5-43f7-afb9-12cb30f6eca3
        name: "T-Mobile Funkadelic 2"
        passthrough:
          connection.type: "bluetooth"
          gsm.apn: "internet2.voicestream.com"
          gsm.device-id: "da812de91eec16620b06cd0ca5cbc7ea25245222"
          gsm.username: "george.clinton.again"
          gsm.sim-operator-id: "310260"
          gsm.pin: "123456"
          gsm.sim-id: "89148000000060671234"
          gsm.password: "parliament2"
          gsm.network-id: "254098"
          ipv4.method: "auto"
          ipv6.method: "auto"''')
        self.assert_nm({'NM-a08c5805-7cf5-43f7-afb9-12cb30f6eca3': '''[connection]
id=T-Mobile Funkadelic 2
#Netplan: passthrough override
type=bluetooth
uuid=a08c5805-7cf5-43f7-afb9-12cb30f6eca3

[gsm]
apn=internet2.voicestream.com
#Netplan: passthrough setting
device-id=da812de91eec16620b06cd0ca5cbc7ea25245222
#Netplan: passthrough setting
username=george.clinton.again
#Netplan: passthrough setting
sim-operator-id=310260
#Netplan: passthrough setting
pin=123456
#Netplan: passthrough setting
sim-id=89148000000060671234
#Netplan: passthrough setting
password=parliament2
#Netplan: passthrough setting
network-id=254098

[ipv4]
#Netplan: passthrough override
method=auto

[ipv6]
#Netplan: passthrough override
method=auto
'''}, '''[device-netplan.modems.NM-a08c5805-7cf5-43f7-afb9-12cb30f6eca3]
match-device=type:gsm
managed=1\n\n''')
        self.assert_networkd({})
        self.assert_nm_udev(None)
